// 12/16/2023  Grove Expansion Board Flash Chip Test Code //
// // Adafruit_SPIFlash Example
// Author: PJG
//
//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Boards 1.1.1
// Board : Seeed XIAO nRF52840 Sense
//----------------------------------------------------------------------------------------------
// SdFat - Adafruit Fork@ 2.2.3
// Adafruit SPIFlash@ 4.1.3
// Adafruit TinyUSB Library@1.7.0
//
// This tests and Identifies the Onboard Xiao QSPI Flash Chip 
// and Expansion Flash Chip on Grove Board Hardwired to A1/D1 (pin 1) on Xiao 
// for the Flash chipSelect CS or SS 
// Adafruit_SPIFlash is set with SPI class mthod. You can mix QSPI mode and SPI class are possible.
// The trick is the Nordic NRF_SPIM0, NRF_SPIM1, NRF_SPIM2, NRF_SPIM3 for which mode or method.
// HTH
// 

#include <Wire.h>
#include <SPI.h>
#include <SdFat.h>
#include <Adafruit_SPIFlash.h>

SPIFlash_Device_t const p25q16h {   // You can Declare the chip you choose this way or add it to the flash_devices
  // Configuration for W25Q80DV flash chip from the Data Sheet.
  .total_size = (1UL << 21), // 2MiB
  .start_up_time_us = 10000,
  .manufacturer_id = 0x85,
  .memory_type = 0x60,
  .capacity = 0x15,
  .max_clock_speed_mhz = 55,
  .quad_enable_bit_mask = 0x02,
  .has_sector_protection = 1,
  .supports_fast_read = 1,
  .supports_qspi = 1,
  .supports_qspi_writes = 1,
  .write_status_register_split = 1,
  .single_status_byte = 0,
  .is_fram = 0,
};
SPIFlash_Device_t const w25q80 {  // Also seemsto be case sensative lower case preferred
  // Configuration for W25Q80DV flash chip from the Data Sheet.
  .total_size = (1UL << 20), // 1MiB
  .start_up_time_us = 5000,
  .manufacturer_id = 0xEF,
  .memory_type = 0x40,
  .capacity = 0x14,
  .max_clock_speed_mhz = 104,
  .quad_enable_bit_mask = 0x02, 
  .has_sector_protection = 0,  
  .supports_fast_read = 1,     
  .supports_qspi = 1,           
  .supports_qspi_writes = 0,    
  .write_status_register_split = 0, 
  .single_status_byte = 0,     
  .is_fram = 0,                 
};

#define SS_SPI0 1  // seems to default to Pin 7 if you don't declare it as the CS for the Flash Chip A1/D1 pin1 on the Xiao
#define SS_SPI1 25

SPIClass SPI_2(NRF_SPIM0, PIN_QSPI_IO1, PIN_QSPI_SCK, PIN_QSPI_IO0); // Onboard QSPI Flash chip
Adafruit_FlashTransport_SPI QflashTransport(PIN_QSPI_CS, SPI_2);     // CS for QSPI Flash

Adafruit_SPIFlash Qflash(&QflashTransport);

SPIClass SPI_1(NRF_SPIM3, D9, D8, D10);   // MISO, SCK, MOSI /EXPANSION FLASH// CS pin D1 (hardwired)
Adafruit_FlashTransport_SPI EflashTransport(SS_SPI0, SPI_1);  // Flash Type

Adafruit_SPIFlash Eflash(&EflashTransport); 


void setup() {
  Serial.begin(9600);
  while (!Serial) delay(2100);
  
  Serial.println();
  Serial.println(" Program " __FILE__ " compiled on " __DATE__ " at " __TIME__ );
  Serial.println("----RESET----");
  pinMode(D6, INPUT_PULLUP);  // /WP
  pinMode(D7, INPUT_PULLUP);  // /HOLD
  pinMode(SS_SPI0, OUTPUT); // CS for flash
  digitalWrite(SS_SPI0, HIGH); // <-- Set CS pin HIGH to deselect
  
  Serial.println("--------------");
  Serial.println(SS_SPI0);
  Serial.println(MOSI); // master out, slave in
  Serial.println(MISO); // master in, slave out
  Serial.println(SCK);  // clock
  Serial.println("--------------");

  Serial.println("Onboard QSPI Flash Testing");
  Serial.println(F("QSPI Flash ID P25Q16H"));
   // Initialize QSPI flash library and check its chip ID.
 if (!Qflash.begin(&p25q16h, 1)) {
   Serial.println(F("Error, failed to initialize QSPI flash chip!"));
   while(1);
  }
  Serial.print(F("QFlash chip JEDEC ID: 0x")); Serial.println(Qflash.getJEDECID(), HEX);
  Serial.print(F("Flash size: ")); Serial.print(Qflash.size() / 1024); Serial.println(F(" KB"));
  Serial.println(F("QFlash chip successfully ID'd!"));
  delay (1500);
  Serial.println();
  Serial.println("Expaansion Flash testing!!");
  digitalWrite(SS_SPI0, HIGH); // <-- Set CS pin HIGH to deselect
  Serial.println(F("SPI Flash ID W25Q80DV"));
   // Initialize W25Q80DV flash library and check its chip ID.
 if (!Eflash.begin(&w25q80)) {
   Serial.println(F("Error, failed to initialize W25Q80DV flash chip!"));
   while(1);
  }
  Serial.print(F("W25Q80DV chip JEDEC ID: 0x")); Serial.println(Eflash.getJEDECID(), HEX);
  Serial.print(F("Flash size: ")); Serial.print(Eflash.size() / 1024); Serial.println(F(" KB"));
  Serial.println(F("SPI Flash chip successfully ID'd!"));
  
}

void loop() {
//delay(500); // Your code here
}
